/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */

package phonebook;

import java.io.BufferedReader;
import java.io.BufferedWriter;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStreamReader;
import java.io.OutputStreamWriter;
import java.io.Writer;
import java.nio.charset.Charset;
import java.util.Collections;
import java.util.Comparator;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;
import java.util.Locale;
import java.util.Map;
import java.util.Set;
import java.util.TreeMap;
import java.util.logging.Level;
import java.util.logging.Logger;


/**
 *
 * @author apple
 */

class PersonComparator implements Comparator<Person>{

    public int compare(Person o1, Person o2) {
        String person1 = o1.getSurname() + o1.getName();
        String person2 = o2.getSurname() + o2.getName();
        return person1.compareToIgnoreCase(person2);
    }

}
public class PhoneBookImpl implements PhoneBook{

    private Map<Person,Set<Phone>> book = new TreeMap<Person,Set<Phone>>(new PersonComparator());

    public void addContact(Person person, Phone contact) {
        if(person == null){
            throw new IllegalArgumentException("person");
        }
        if(contact == null){
            throw new IllegalArgumentException("contact");
        }

        Set<Phone> phones;

        if(book.containsKey(person)){
            phones = book.get(person);
        }else{
            phones = new HashSet<Phone>();
        }
        phones.add(contact);
        book.put(person, phones);
    }

    public Set<Phone> contacts(Person person) {
        return Collections.unmodifiableSet(book.get(person));
    }

    public Set<Person> persons() {
        return Collections.unmodifiableSet(book.keySet());
    }

    public void store(String filename) throws PhoneBookException {
        Writer output = null;

        try {
            output = new BufferedWriter(new OutputStreamWriter(new FileOutputStream(filename), Charset.defaultCharset()));
        } catch (FileNotFoundException ex) {
            throw new PhoneBookException("Nelze otevrit soubor.",ex);
        }

        for(Person person:book.keySet()){
            try {
                output.write(person.getName() + " " + person.getSurname() + "\n");
            } catch (IOException ex) {
                try {
                    output.close();
                    throw new PhoneBookException("Nelze nacist data",ex);
                } catch (IOException ex1) {
                    throw new PhoneBookException("Nelze zavrit vystupni soubor",ex1);
                }
            }
            try {
                writeContacts(book.get(person), output);
            } catch (IOException ex) {
                try{
                    output.close();
                    throw new PhoneBookException("Nelze zapsat kontakty.",ex);
                }catch(IOException ex1){
                    throw new PhoneBookException("Nelze zavrit soubor po neuspesnem pokusu o zapis kontaktu.",ex1);
                }
            }
        }
        try {
            output.close();
        } catch (IOException ex) {
            throw new PhoneBookException("Nelze zavrit vystup");
        }
    }

    public void load(String filename) throws PhoneBookException {
        BufferedReader input;

        try {
            input = new BufferedReader(new InputStreamReader(new FileInputStream(filename), Charset.defaultCharset()));
        } catch (FileNotFoundException ex) {
            throw new PhoneBookException("Nelze otevrit vstup.", ex);
        }

        String line=null;
        try {
            line = input.readLine();
        } catch (IOException ex) {
            try {
                input.close();
                throw new PhoneBookException("Nelze nacist radek",ex);
            } catch (IOException ex1) {
                throw new PhoneBookException("Nelze zavrit vstup po neuspesnem nacitani radku.",ex1);
            }
        }

        while(line != null){
            String personString[] = line.split(" ");
            if(personString.length<2){
                throw new PhoneBookException("Nekorektni jmeno osoby: "+line);
            }

            Person person = new PersonImpl(personString[0], personString[1]);
            Set<Phone> contacts;
            try {
                contacts = getContacts(input);
            } catch (IOException ex) {
                try {
                    input.close();
                    throw new PhoneBookException("Format kontaktu:", ex);
                } catch (IOException ex1) {
                    throw new PhoneBookException("Zavirani souboru po chybe formatu kontaktu.",ex1);
                }
            }
            try {
                line = input.readLine();
            } catch (IOException ex) {
                try{
                    input.close();
                    throw new PhoneBookException("Nelze nacist dalsi jmeno osoby.", ex);
                }catch(IOException ex1){
                    throw new PhoneBookException("Nelze zavrit vstup po neuspesnem cteni:", ex1);
                }
            }
            book.put(person,contacts);
        }
        try {
            input.close();
        } catch (IOException ex) {
            throw new PhoneBookException("Nelze zavrit vstup.", ex);
        }
    }

    private Set<Phone> getContacts(BufferedReader input) throws IOException {
        Set<Phone> contacts = new HashSet<Phone>();
        String line;
        line = input.readLine();
        String strContacts[] = line.split(";");
        if(strContacts.length < 1){
            throw new IOException("Chybi kontakty.");
        }

        for(String contact:strContacts){
           String[] strContact = contact.split(":");
           if(strContact.length != 2 ){
               throw new IOException("Spatny format kontaktu:" + strContact);
           }
           Phone phone = new PhoneImpl(strContact[0],strContact[1]);
           contacts.add(phone);
        }

        return contacts;
    }

    private void writeContacts(Set<Phone> phones, Writer output) throws IOException {
        StringBuilder line = new StringBuilder();
        for(Iterator<Phone> it=phones.iterator();it.hasNext();){
            Phone phone = it.next();
            line.append(phone.getAddress()+":"+phone.getNumber());
            if(it.hasNext()){
                line.append(";");
            }
        }

        output.write(line.toString() + "\n");

    }

}
